@@ -13,7 +13,6 @@ import weakref
 from collections import defaultdict, namedtuple
 from itertools import islice
 from typing import List, Callable, TypeVar, Union, Tuple, overload, Mapping, Dict
-
 import numpy as np
 from distutils.version import Version, LooseVersion
 import tensorflow as tf
@@ -21,19 +20,29 @@ from tensorflow.python import enable_eager_execution
 from tensorflow.python.eager import context
 from tensorflow.python.framework import func_graph, ops
 from tensorflow.python.training.tracking import base as trackable
-from tensorflow.python.training.tracking import data_structures,tracking
+from tensorflow.python.training.tracking import data_structures, tracking
 from tensorflow.python.training.tracking import layer_utils as trackable_layer_utils
 
 from tensorflow.python.module import module
 from tensorflow.python.util import object_identity
 from tensorflow.python.keras.engine import base_layer_utils
 from trident.backend import tensorflow_serialization as serialization
-from trident.backend.common import camel2snake, to_list, unpack_singleton, enforce_singleton, OrderedDict, get_signature,get_session,set_session
+from trident.backend.common import camel2snake, to_list, unpack_singleton, enforce_singleton, OrderedDict, get_session, set_session, Signature
+from trident.backend.tensorspec import *
+from trident.backend import tensorflow_ops as tops
 from trident.backend.tensorflow_ops import *
 from trident.data.utils import pickle_it
 
+__all__ = ['set_device','Layer', 'get_device', 'get_flops', 'Sequential', 'summary', 'normalize_padding', 'load', 'save', 'try_map_args_and_call', 'fix_layer']
 
-__all__ = ['Layer','get_device', 'get_flops', 'Sequential', 'ReplayBuffer', 'summary', 'normalize_padding', 'load', 'save','try_map_args_and_call']
+_FUN_NAMES = [
+    ('float', tops.float),
+    ('long', tops.long),
+    ('int', tops.int),
+    ('to', tops.to)]
+for target_fun_name, source_fun in _FUN_NAMES:
+    if not hasattr(Tensor, target_fun_name):
+        setattr(Tensor, target_fun_name, source_fun)
 
 
 def get_device():
@@ -43,16 +52,16 @@ def get_device():
 
     """
     if get_session().device is None:
-        set_device('/gpu:0' if  len(tf.config.list_physical_devices('GPU'))>0 else "/cpu:0")
+        set_device('/gpu:0' if len(tf.config.list_physical_devices('GPU')) > 0 else "/cpu:0")
     return get_session().device
 
 
 def set_device(device='/cpu:0'):
-    if device.lower()=='cuda' or device.lower()=='gpu':
-        device='/gpu:0'
-    if device.lower()=='cpu':
-        device='/cpu:0'
-    if 'gpu' in device and len(tf.config.list_physical_devices('GPU'))==0:
+    if device.lower() == 'cuda' or device.lower() == 'gpu':
+        device = '/gpu:0'
+    if device.lower() == 'cpu':
+        device = '/cpu:0'
+    if 'gpu' in device and len(tf.config.list_physical_devices('GPU')) == 0:
         raise ValueError('Gpu is not available...')
     try:
         set_session('device', device)
@@ -61,16 +70,13 @@ def set_device(device='/cpu:0'):
         print(e)
 
 
-
-
 version = tf.version
 sys.stdout.write('Tensorflow version:{0}.\n'.format(version.VERSION))
 
+tf_version = LooseVersion(vstring=version.VERSION)
+base_version = LooseVersion(vstring='2.2.0-rc0')
 
-tf_version=LooseVersion(vstring=version.VERSION)
-base_version=LooseVersion(vstring='2.2.0-rc0')
-
-if tf_version.version <base_version.version:
+if tf_version.version < base_version.version:
     raise ValueError('trident only support Tensorflow 2.2.0-rc0 or newer.\n')
 
 sys.stdout.write('use device:{0}.\n'.format(get_device()))
@@ -95,7 +101,7 @@ def load(path):
     if '.tar' in path:
         return serialization.load_pthtar(path)
     else:
-        return  serialization.load(path)
+        return serialization.load(path)
 
 
 def save(obj, path, is_compressed=False):
@@ -182,14 +188,18 @@ def reset_name(module: tf.Module, prefix_dict=None):
             module._uid_prefixs[prefix] = seq
         return module._uid_prefixs[prefix]
 
+
     if not hasattr(module, '_uid_prefixs') or prefix_dict is not None:
         module._uid_prefixs = prefix_dict
-    if not hasattr(module, 'default_name'):
-        module.default_name = camel2snake(module.__class__.__name__) + '_' + str(
-            get_global_uid(camel2snake(module.__class__.__name__)))
-    prefix, seq = module.default_name.rsplit('_', 1)  # if '_' in module._default_name else
+    if not hasattr(module, '_default_name'):
+        module._default_name = camel2snake(module.__class__.__name__) + '_' + str(get_global_uid(camel2snake(module.__class__.__name__)))
+    prefix, seq = module._default_name.rsplit('_', 1)  # if '_' in module._default_name else
     seq = int(seq)
-    module._default_name = prefix + '_' + str(seq - get_uid(prefix, seq) + 1)
+    module.default_name = prefix + '_' + str(seq - get_uid(prefix, seq) + 1)
+    if module._name  is None:
+        module._name=module.default_name
+    module.__name__ = module._name
+    module.update_name_scope(module._name)
 
 
 _UID_PREFIX = defaultdict(int)
@@ -199,7 +209,6 @@ def get_global_uid(prefix=''):
     _UID_PREFIX[prefix] += 1
     return _UID_PREFIX[prefix]
 
-
 class _IncompatibleKeys(namedtuple('IncompatibleKeys', ['missing_keys', 'unexpected_keys'])):
     def __repr__(self):
         if not self.missing_keys and not self.unexpected_keys:
@@ -221,9 +230,6 @@ def _addindent(s_, numSpaces):
     return s
 
 
-
-
-
 r"""This tracks hooks common to all modules that are executed before/after
 calling forward and backward. This is global state used for debugging/profiling
 purposes"""
@@ -237,6 +243,7 @@ _grad_t = Union[Tuple[Tensor, ...], Tensor]
 # the type of the subclass, not the looser type of `Module`.
 T = TypeVar('T', bound='Layer')
 
+
 def register_module_forward_pre_hook(hook: Callable[..., None]) -> RemovableHandle:
     r"""Registers a forward pre-hook common to all modules.
 
@@ -296,12 +303,13 @@ def register_module_forward_hook(hook: Callable[..., None]) -> RemovableHandle:
     This hook will be executed before specific module hooks registered with
     ``register_forward_hook``.
     """
-    handle =RemovableHandle(_global_forward_hooks)
+    handle = RemovableHandle(_global_forward_hooks)
     _global_forward_hooks[handle.id] = hook
     return handle
 
+
 def register_module_backward_hook(
-    hook: Callable[['Layer', _grad_t, _grad_t], Union[None, Tensor]]
+        hook: Callable[['Layer', _grad_t, _grad_t], Union[None, Tensor]]
 ) -> RemovableHandle:
     r"""Registers a backward hook common to all the modules.
 
@@ -336,15 +344,11 @@ def register_module_backward_hook(
             ``handle.remove()``
 
     """
-    handle =RemovableHandle(_global_backward_hooks)
+    handle = RemovableHandle(_global_backward_hooks)
     _global_backward_hooks[handle.id] = hook
     return handle
 
 
-
-
-
-
 class Layer(tf.Module):
     """Trident extened tf.Module as base layer class.
 
@@ -389,7 +393,7 @@ class Layer(tf.Module):
     """
     _version = 1
 
-    def __init__(self, name=None, keep_output=False,**kwargs):
+    def __init__(self, name=None, keep_output=False, **kwargs):
         """
         Args:
             name (str) :name of the layer.
@@ -398,27 +402,25 @@ class Layer(tf.Module):
 
         """
         self._uid_prefixs = {}
-        self._name = kwargs.get('name', name)
+        self._name = name
         self.uuid = uuid.uuid4().node
         prefix = self.__class__.__name__
-        self.default_name = camel2snake(prefix) + '_' + str(get_global_uid(camel2snake(prefix)))
-        #reset_name(self, self._uid_prefixs)
-        self.relative_name = ''
 
-        super(Layer, self).__init__(name=self.default_name )
+        self._default_name = camel2snake(prefix) + '_' + str(get_global_uid(camel2snake(prefix)))
+        self.relative_name = ''
+        #reset_name(self, self._uid_prefixs)
+        super(Layer, self).__init__(name=self._default_name)
 
         self._nodes = None
 
         with self.name_scope:
             self.batch_index = 0
             self.filter_index = -1
-            self.in_sequence = False
+            self.in_sequence = kwargs.get('in_sequence', False)
             self.training = True
             self._built = False
             self.rank = kwargs.get('rank', None)
 
-
-
             self._modules = OrderedDict()
             self._parameters = OrderedDict()
             self._buffers = OrderedDict()
@@ -432,29 +434,30 @@ class Layer(tf.Module):
 
             self._input_shape = None
             self.input_filters = None
+            self.input_spec = None
             self._output_shape = None
             self.keep_output = keep_output
             self._output_tensor = None
-            self.signature = None
-
-
-
+            self._signature = None
 
             # self.dump_patches = True
 
             self._device = get_device()
 
+    @property
+    def name(self):
+        """Name of the layer (string), set in the constructor."""
+        return self._name
 
-    # @property
-    # def name(self):
-    #     """If not assign name , it will return the default_name"""
-    #     return self._name if self._name is not None and len(self._name) > 0 else self.default_name
-    #
-    # @name.setter
-    # def name(self, value):
-    #     self._name = value
+    @name.setter
+    def name(self, value):
+        self._name = value
 
     # def _set_name_scope(self):
+    def update_name_scope(self,name):
+        self._name = name
+        with ops.name_scope_v2(name) as scope_name:
+            self._name_scope = ops.name_scope_v2(scope_name)
 
 
     @property
@@ -496,28 +499,27 @@ class Layer(tf.Module):
             raise KeyError("module name can't contain \".\"")
         elif name == '':
             raise KeyError("module name can't be empty string \"\"")
-        elif isinstance(module, Layer):
-            self._modules[name] = module
-            if isinstance(module, Layer):
-                reset_name(module, self._uid_prefixs)
-                module.relative_name = name if module.relative_name == '' else name + '.' + module.relative_name
-                for mod in module.modules():
-                    if isinstance(mod, Layer) and mod.uuid != module.uuid:
-                        mod.nodes = self.nodes
-                        reset_name(mod, self._uid_prefixs)
-                        mod.relative_name = name if mod.relative_name == '' else name + '.' + mod.relative_name
 
-            self.nodes = OrderedDict([(mod.uuid, mod) for mod in list(self.modules()) if isinstance(mod, Layer)])
+        self._modules[name] = module
+        if isinstance(module, Layer):
+            reset_name(module, self._uid_prefixs)
+            module.relative_name = name if module.relative_name == '' else name + '.' + module.relative_name
             for mod in module.modules():
-                if isinstance(mod, Layer):
+                if isinstance(mod, Layer) and mod.uuid != module.uuid:
                     mod.nodes = self.nodes
+                    reset_name(mod, self._uid_prefixs)
+                    mod.relative_name = name if mod.relative_name == '' else name + '.' + mod.relative_name
+
+        self.nodes = OrderedDict([(mod.uuid, mod) for mod in list(self.modules()) if isinstance(mod, Layer)])
+        for mod in self.modules():
+            if isinstance(mod, Layer):
+                mod.nodes = self.nodes
+        #
+        # elif inspect.isfunction(module) or callable(module):
+        #     module.__name__ = name
+        #     self._modules[name] = module
 
-        elif inspect.isfunction(module) or callable(module):
-            module.__name__ = name
-            self._modules[name] = module
 
-        else:
-            raise ValueError('Not valid module')
 
     def add(self, module):
         """Simplified 'add_module'
@@ -534,12 +536,11 @@ class Layer(tf.Module):
             # nodes = self._nodes.copy()
             # for k, v in module.nodes.items():
             #     nodes[k] = v
-            self.add_module(str(len(self._modules)),module)  # self.nodes = nodes  # for mod in self.modules():  #     mod.nodes = nodes
+            self.add_module(str(len(self._modules)), module)  # self.nodes = nodes  # for mod in self.modules():  #     mod.nodes = nodes
 
         else:
             raise ValueError('Not valid module')
 
-
     def build(self, input_shape):
         """ Do the shape inference and initialize weights and bias.
 
@@ -581,7 +582,7 @@ class Layer(tf.Module):
             directly on a specific input or output to get the required gradients.
 
         """
-        handle =RemovableHandle(self._backward_hooks)
+        handle = RemovableHandle(self._backward_hooks)
         self._backward_hooks[handle.id] = hook
         return handle
 
@@ -706,7 +707,6 @@ class Layer(tf.Module):
         else:
             self._parameters[name] = param
 
-
     def cuda(self, device=None):
         r"""Moves all model parameters and buffers to the GPU.
 
@@ -772,8 +772,6 @@ class Layer(tf.Module):
         fn(self)
         return self
 
-
-
     @property
     def device(self):
         return self._device
@@ -788,25 +786,21 @@ class Layer(tf.Module):
 
     @input_shape.setter
     def input_shape(self, value):
-        if isinstance(value, (list, tuple)) and len(value) > 0:
-            value = tf.TensorShape(list(value))
-        elif is_tensor(value):
-            value = to_numpy(value).tolist()
-        elif isinstance(value, int):  #
-            value = tf.TensorShape(value)
-        elif isinstance(value, np.ndarray) and value.ndim <= 1:
-            value = tf.TensorShape(value.astype(np.uint8).tolist())
-        elif isinstance(value, tf.TensorShape):
-            value = value
-        else:
-            raise ValueError('not valid input_shape')
+        if isinstance(value, tf.TensorShape):
+            value = to_tensor(value.as_list()).to('int')
+        elif isinstance(value, (list, tuple)) and len(value) > 0:
+            value = tuple([to_tensor(tensor_shape.as_list()).to('int') if isinstance(tensor_shape, tf.TensorShape) else to_tensor(tensor_shape).to('int') for tensor_shape in value])
 
-        if self._built == False:
+        else:
+            value = to_tensor(value).to('int')
+
+        self.input_spec = TensorSpec(shape=value)
+        if self._built == False or self._input_shape is None:
             self._input_shape = value
             if len(self._input_shape) == 0:
-                self.input_filters = int(to_numpy(self._input_shape).data)
+                self.input_filters = to_numpy(self._input_shape)[self.filter_index]
             elif len(self._input_shape) == 1:
-                self.input_filters = int(to_numpy(self._input_shape)[-1])
+                self.input_filters = to_numpy(self._input_shape)[self.filter_index]
             else:
                 if self.filter_index < 0:
                     self.input_filters = int(self._input_shape[self.filter_index])
@@ -817,9 +811,10 @@ class Layer(tf.Module):
 
             self.build(self._input_shape)
             self._built = True
+            self._signature = None
 
 
-        elif self._input_shape is not None and self._input_shape == tuple(to_list(value)):
+        elif self._input_shape is not None and to_list(self._input_shape) == to_list(value):
             'input_shape is already assigned, and shape is the same.'
             pass
 
@@ -829,19 +824,41 @@ class Layer(tf.Module):
 
     @output_shape.setter
     def output_shape(self, value):
-        if isinstance(value, (list, tuple)) and len(value) > 0:
-            value = tf.TensorShape(list(value))
-        elif is_tensor(value):
-            value = tf.TensorShape(to_numpy(value).tolist())
-        elif isinstance(value, int):  #
-            value = tf.TensorShape(value)
-        elif isinstance(value, np.ndarray) and value.ndim <= 1:
-            value = tf.TensorShape(value.astype(np.uint8).tolist())
-        elif isinstance(value, tf.TensorShape):
-            value = value
+        if isinstance(value, tf.TensorShape):
+            value = to_tensor(value.as_list()).to('int')
+        elif isinstance(value, (list, tuple)) and len(value) > 0:
+            value = tuple([to_tensor(tensor_shape.as_list()).to('int') if isinstance(tensor_shape, tf.TensorShape) else to_tensor(tensor_shape).to('int') for tensor_shape in value])
+
         else:
-            raise ValueError('not valid input_shape')
+            value = to_tensor(value).to('int')
         self._output_shape = value
+        self._signature = None
+
+    @property
+    def signature(self):
+        if self._signature is None or len(self._signature) == 0:
+            self._signature = Signature(name=self.name)
+            if self._input_shape is not None:
+                if is_tensor(self._input_shape):
+                    self._signature.inputs["input"] = TensorSpec(shape=to_tensor(self._input_shape), name="input")
+                elif isinstance(self._input_shape, tuple) and isinstance(self._input_shape[0], int):
+                    self._signature.inputs["input"] = TensorSpec(shape=to_tensor(self._input_shape), name="input")
+                elif isinstance(self._input_shape, tuple):
+                    for i in range(len(self._input_shape)):
+                        self._signature.inputs["input_{0}".format(i)] = TensorSpec(shape=to_tensor(self._input_shape[i]), name="input_{0}".format(i))
+            if self._output_shape is not None:
+                if is_tensor(self._output_shape):
+                    self._signature.outputs["output"] = TensorSpec(shape=self._output_shape, name="output")
+                elif isinstance(self._output_shape, tuple) and isinstance(self._output_shape[0], int):
+                    self._signature.outputs["output"] = TensorSpec(shape=to_tensor(self._output_shape), name="output")
+                elif isinstance(self._output_shape, tuple):
+                    for i in range(len(self._output_shape)):
+                        self._signature.outputs["output_{0}".format(i)] = TensorSpec(shape=to_tensor(self._output_shape[i]), name="output_{0}".format(i))
+        return self._signature
+
+    @signature.setter
+    def signature(self, value):
+        self._signature = value
 
     @property
     def input(self):
@@ -883,6 +900,7 @@ class Layer(tf.Module):
     @overload
     def state_dict(self, prefix: str = ..., keep_vars: bool = ...) -> Dict[str, Tensor]:
         ...
+
     def _save_to_state_dict(self, destination, prefix, keep_vars):
         r"""Saves module state to `destination` dictionary, containing a state
         of the module, but not its descendants. This is called on every
@@ -938,7 +956,6 @@ class Layer(tf.Module):
         self._load_state_dict_pre_hooks[handle.id] = hook
         return handle
 
-
     def _load_from_state_dict(self, state_dict, prefix, local_metadata, strict,
                               missing_keys, unexpected_keys, error_msgs):
         r"""Copies parameters and buffers from :attr:`state_dict` into only
@@ -1003,7 +1020,7 @@ class Layer(tf.Module):
                                       'whose dimensions in the model are {} and '
                                       'whose dimensions in the checkpoint are {}, '
                                       'an exception occured : {}.'
-                                      .format(key, param.size(), input_param.size(), ex.args))
+                                      .format(key, numel(param), numel(input_param), ex.args))
             elif strict:
                 missing_keys.append(key)
 
@@ -1060,9 +1077,9 @@ class Layer(tf.Module):
                 error_msgs.insert(0, 'Missing key(s) in state_dict: {}. '.format(
                     ', '.join('"{}"'.format(k) for k in missing_keys)))
 
-        if len(error_msgs) > 0:
-            raise RuntimeError(
-                'Error(s) in loading state_dict for {}:\n\t{}'.format(self.__class__.__name__, "\n\t".join(error_msgs)))
+        # if len(error_msgs) > 0:
+        #     raise RuntimeError(
+        #         'Error(s) in loading state_dict for {}:\n\t{}'.format(self.__class__.__name__, "\n\t".join(error_msgs)))
         return _IncompatibleKeys(missing_keys, unexpected_keys)
 
     def save(self, file_path=''):
@@ -1091,7 +1108,8 @@ class Layer(tf.Module):
     def _slow_forward(self, *input, **kwargs):
         return self.forward(*input, **kwargs)
         # tracing_state = torch._C._get_tracing_state()  # if not tracing_state or isinstance(self.forward, torch._C.ScriptMethod):  #     return self.forward(*input, **kwargs)  # recording_scopes = torch.jit._trace_module_map is not None  # if recording_scopes:  #     name = torch.jit._trace_module_map[self] if self in torch.jit._trace_module_map else None  #     if name:  #         cur_scope_name = tracing_state.current_scope()  #         tracing_state.push_scope(name)  #     else:  #         recording_scopes = False  # try:  #     result = self.forward(*input, **kwargs)  # finally:  #     if recording_scopes:  #         tracing_state.pop_scope()  # return result
-        #tracing_state = torch._C._get_tracing_state()
+
+        # tracing_state = torch._C._get_tracing_state()
         # if not tracing_state or isinstance(self.forward, torch._C.ScriptMethod):
         #     return self.forward(*input, **kwargs)
         # recording_scopes = torch.jit._trace_module_map is not None
@@ -1109,11 +1127,10 @@ class Layer(tf.Module):
         #         tracing_state.pop_scope()
         # return result
 
-
     @tf.Module.with_name_scope
     def __call__(self, *input, **kwargs):
         # Maintains info about the `Layer.call` stack.
-        is_all_numpy=True
+        is_all_numpy = True
         input = list(input)
         new_input = []
         for inp in input:
@@ -1125,21 +1142,21 @@ class Layer(tf.Module):
                 is_all_numpy = False
         input = new_input
         for hook in self._forward_pre_hooks.values():
-            result  = hook(self, *input)
+            result = hook(self, *input)
             if result is not None:
                 if not isinstance(result, tuple):
                     result = (result,)
                 input = result
-        if self._built==False :
-            inp= unpack_singleton(input)
+        if self._built == False:
+            inp = unpack_singleton(input)
             if isinstance(inp, (tuple, list)):
-                self.build([input_tensor.shape[self.batch_index+1:] for input_tensor in inp])
+                self.build([to_tensor(int_shape(input_tensor)[self.batch_index + 1:]) for input_tensor in inp])
             elif is_tensor(inp):
-                self.input_shape = inp.shape[self.batch_index+1:]
+                self.input_shape = to_tensor(int_shape(inp)[self.batch_index + 1:])
             else:
                 print('input shou be tensor or tuple of tensor')
                 print(inp)
-                self.input_shape= tf.TensorShape(None)
+                self.input_shape = tf.TensorShape(None)
         # don't use result = self.forward(i*nput, **kwargs) because EagerTensor will splited as a tuple....
         try:
             with tf.device(get_device()):
@@ -1147,9 +1164,11 @@ class Layer(tf.Module):
                 output = unpack_singleton(result)
                 if hasattr(self, 'keep_output') and self.keep_output == True:
                     self._output_tensor = output
-                if is_tensor(output):
-                    if self._output_shape is None:
-                        self.output_shape = output.shape[self.batch_index+1:]
+                if isinstance(output, Tensor):  # one output
+                    if self._output_shape is None or not np.array_equal(to_numpy(self._output_shape) ,to_numpy(int_shape(output))[self.batch_index + 1:]):
+                        self._output_shape = to_tensor(int_shape(output)[self.batch_index + 1:])
+                elif isinstance(output, (list, tuple)):
+                    self._output_shape = tuple([to_tensor(int_shape(output_tensor)[self.batch_index + 1:] )for output_tensor in output])
 
             for hook in self._forward_hooks.values():
                 hook_result = hook(self, input, result)
@@ -1667,23 +1686,61 @@ class Sequential(Layer):
 
         # self.to(self.device)
 
+    # @property
+    # def output_shape(self):
+    #     if len(self)>0:
+    #         return self[-1]._output_shape
+    #     else:
+    #         return None
+    #
+    # @output_shape.setter
+    # def output_shape(self, value):
+    #     if len(self) > 0:
+    #         if isinstance(value, tf.TensorShape):
+    #             value = to_tensor(value.as_list()).to('int')
+    #         elif isinstance(value, (list, tuple)) and len(value) > 0:
+    #             value = tuple(
+    #                 [to_tensor(tensor_shape.as_list()).to('int') if isinstance(tensor_shape, tf.TensorShape) else to_tensor(tensor_shape).to('int') for tensor_shape in value])
+    #
+    #         else:
+    #             value = to_tensor(value).to('int')
+    #         self[-1]._output_shape = value
+    #         self._signature=None
+
     def build(self, input_shape):
         if self._built == False and len(self._modules) > 0:
             self.__getitem__(0).input_shape = self.input_shape
             self._built = True
 
-    def sync_build(self):
-        input_shape = None
-        # if self[:1] is Input:
-        #     input_shape=self[:1].input_shape
-        if input_shape is not None:
-            input_shape = list(input_shape)
-            input_shape.insert(0, 2)
-            data = to_tensor(np.random.standard_normal(input_shape))
-            out = self(data)
+    def add_module(self, name, module):
+        r"""Adds a child module to the current module.
+
+        The module can be accessed as an attribute using the given name.
+
+        Args:
+            name (string): name of the child module. The child module can be
+                accessed from this module using the given name
+            module (Module): child module to be added to the module.
+        """
+
+        if len(self._modules) > 0 and self._input_shape is not None and self[-1].built and self[-1]._output_shape is not None:
+            last_output = self[-1]._output_shape
+            super(Sequential, self).add_module(name, module)
+            dummay_input = random_normal((2,) + tuple(to_list(last_output))).to(self.device)
+            out = module(dummay_input)
+            self._output_shape = module._output_shape
+            self._signature=None
+        else:
+            super(Sequential, self).add_module(name, module)
+            self._signature = None
+
+        self._signature = None
 
     def remove_at(self, idx):
         self.__delitem__(idx)
+        if len(self._modules) > 0:
+            self._output_shape = self[-1]._output_shape
+            self._signature = None
 
     def _get_item_by_idx(self, iterator, idx):
         """Get the idx-th item of the iterator"""
@@ -1724,13 +1781,13 @@ class Sequential(Layer):
         return keys
 
     def forward(self, *x):
-        x = enforce_singleton(x)
         for module in self._modules.values():
+            x = enforce_singleton(x)
             x = module(x)
         return x
 
 
-class Combine(tf.keras.Model):
+class Combine(Layer):
     r"""A sequential container.
     Modules will be added to it in the order they are passed in the constructor.
     Alternatively, an ordered dict of modules can also be passed in.
@@ -1754,68 +1811,64 @@ class Combine(tf.keras.Model):
                 ]))
     """
 
-    def __init__(self, *args):
+    def __init__(self, *args, name=None):
         super(Combine, self).__init__()
+        self._name = name
         self._built = False
 
         if len(args) == 1 and isinstance(args[0], OrderedDict):
             for key, module in args[0].items():
-                module.__name__ = key
-                self._layers.append(module)
+                self.add_module(key, module)
         elif len(args) == 1 and isinstance(args[0], (list)):
             for idx, module in enumerate(args[0]):
-                self._layers.append(module)
+                self.add_module(str(idx), module)
         else:
             for idx, module in enumerate(args):
-                self._layers.append(module)
+                self.add_module(str(idx), module)
+        self.to(self.device)
 
-    @property
-    def layers(self):
-        return self._layers
+    def _get_item_by_idx(self, iterator, idx):
+        """Get the idx-th item of the iterator"""
+        size = len(self)
+        idx = idx.__index__()
+        if not -size <= idx < size:
+            raise IndexError('index {} is out of range'.format(idx))
+        idx %= size
+        return next(islice(iterator, idx, None))
+
+    def __getitem__(self, idx):
+        if isinstance(idx, slice):
+            return self.__class__(OrderedDict(list(self._modules.items())[idx]))
+        else:
+            return self._get_item_by_idx(self._modules.values(), idx)
+
+    def __setitem__(self, idx, module):
+        key = self._get_item_by_idx(self._modules.keys(), idx)
+        return setattr(self, key, module)
+
+    def __delitem__(self, idx):
+        if isinstance(idx, slice):
+            for key in list(self._modules.keys())[idx]:
+                delattr(self, key)
+        else:
+            key = self._get_item_by_idx(self._modules.keys(), idx)
+            delattr(self, key)
 
     def __len__(self):
-        return len(self._layers)
+        return len(self._modules)
 
     def __dir__(self):
         keys = super(Combine, self).__dir__()
         keys = [key for key in keys if not key.isdigit()]
         return keys
 
-    def call(self, inputs, training=None, mask=None):  # pylint: disable=redefined-outer-name
-        if self._is_graph_network:
-            if not self.built:
-                self._init_graph_network(self.inputs, self.outputs, name=self.name)
-            return super(Combine, self).call(inputs, training=training, mask=mask)
-
+    def forward(self, *x):
         outputs = []
-        for layer in self._layers:
-            outputs.append(layer(inputs))
+        for module in self._modules.values():
+            outputs.append(module(*x))
         return tuple(outputs)
 
 
-class ReplayBuffer:
-    def __init__(self, max_size=1000):
-        assert max_size > 0, "Empty buffer or trying to create a black hole. Be careful."
-        self.max_size = max_size
-        self.data = []
-
-    def push_and_pop(self, data):
-        to_return = []
-        for element in data.data:
-            element = tf.keras.backend.expand_dims(element, 0)
-            if len(self.data) < self.max_size:
-                self.data.append(element)
-                to_return.append(element)
-            else:
-                if random.uniform(0, 1) > 0.5:
-                    i = random.randint(0, self.max_size - 1)
-                    to_return.append(self.data[i].clone())
-                    self.data[i] = element
-                else:
-                    to_return.append(element)
-        return tf.concat(to_return, axis=-1)
-
-
 def count_params(weights):
     """Count the total number of scalars composing the weights.
 
@@ -1854,24 +1907,22 @@ def calculate_flops(gen: Layer):
     return np.array(param_nums).sum()
 
 
-
-
-
 def summary(model, input_size, batch_size=-1):
     def register_hook(module):
         def hook(module, input, output):
             class_name = str(module.__class__).split(".")[-1].split("'")[0]
             module_idx = len(summary)
 
-            m_key = module.name
+            m_key =module.default_name if  module._name is None else module._name
+            #m_key = module.name
             summary[m_key] = OrderedDict()
             summary[m_key]["keep_output"] = module.keep_output
-            summary[m_key]["input_shape"] = list(to_numpy(input[0]).shape)
+            summary[m_key]["input_shape"] =  list(int_shape(input[0]))
             summary[m_key]["input_shape"][0] = batch_size
             if isinstance(output, (list, tuple)):
                 summary[m_key]["output_shape"] = [[-1] + list(o.size())[1:] for o in output]
             else:
-                summary[m_key]["output_shape"] = list(to_numpy(output).shape)
+                summary[m_key]["output_shape"] = list(int_shape(output))
                 summary[m_key]["output_shape"][0] = batch_size
 
             params = 0
@@ -1941,17 +1992,17 @@ def summary(model, input_size, batch_size=-1):
         # input_shape, output_shape, trainable, nb_params
         is_keep = '★' if summary[layer]["keep_output"] else ''
         line_new = "{0:<40s} {1:<20s}  {2:^20s} {3:^8s}  {4:^8}  {5:^12}".format(layer,
-            is_keep + str(summary[layer]["output_shape"]),
-            str(summary[layer]["weight"] if 'weight' in summary[layer] else ''),
-            str(summary[layer]["bias"] if 'bias' in summary[layer] else ''), summary[layer]["nb_params"],
-            summary[layer]["flops"][0])
+        is_keep + str(summary[layer]["output_shape"]),
+       str(summary[layer]["weight"] if 'weight' in summary[layer] else ''),
+       str(summary[layer]["bias"] if 'bias' in summary[layer] else ''),
+       summary[layer]["nb_params"],
+        summary[layer]["flops"][0])
         total_params += summary[layer]["nb_params"]
-        flops += float(summary[layer]["flops"])
+        flops += float(summary[layer]["flops"][0])
         macc += float(summary[layer]["macc"][0])
         total_output += np.prod(to_numpy(summary[layer]["output_shape"]))
         if "trainable" in summary[layer]:
-            if summary[layer]["trainable"] == True:
-                trainable_params += summary[layer]["nb_params"]
+            trainable_params += int(summary[layer]["trainable"])
         print(line_new)
 
     # assume 4 bytes/number (float on cuda).
@@ -2038,7 +2089,7 @@ def normalize_padding(padding, rank):
     return padding
 
 
-def try_map_args_and_call(fn, data: OrderedDict, data_feed=None,grad_tape=None):
+def try_map_args_and_call(fn, data: OrderedDict, data_feed: OrderedDict = None, grad_tape=None):
     """This function is the core function for mapping callable and argments
 
     Args:
@@ -2110,10 +2161,6 @@ def try_map_args_and_call(fn, data: OrderedDict, data_feed=None,grad_tape=None):
             print('uncomplete arg_map', arg_map.key_list)
 
 
-
-
-
-
 def force_deterministic(seed):
     """ Force most of the computation nodes to run deterministically.
 
@@ -2129,6 +2176,94 @@ def force_deterministic(seed):
     tf.config.threading.set_inter_op_parallelism_threads(1)
 
 
+def fix_layer(layer: Layer):
+    """fix existing out-of-date model compatibility
 
+    Args:
+        layer (trident Layer):
+
+    Returns: fixed layer
+
+    """
+
+    if layer._input_shape is not None and isinstance(layer._input_shape, tf.TensorShape):
+        layer._input_shape = to_tensor(layer._input_shape.as_list()).to('int')
+    if layer._output_shape is not None and isinstance(layer._output_shape, tf.TensorShape):
+        layer._output_shape = to_tensor(layer._output_shape.as_list()).to('int')
+
+    for module in layer.modules():
+        class_name = module.__class__.__name__
+        if not hasattr(module, '_uid_prefixs'):
+            module._uid_prefixs = {}
+        if not hasattr(module, '_name'):
+            module._name = None
+            reset_name(module, module._uid_prefixs)
+
+        if not hasattr(module, 'name'):
+            module.name = module._name if module._name is not None and len(module._name) > 0 else module.relative_name
+
+        if not hasattr(module, '_built'):
+            setattr(module, 'built', True)
+
+        if hasattr(module, 'keepdim'):
+            value = getattr(module, 'keepdim')
+            delattr(module, 'keepdim')
+            setattr(module, 'keepdims', value)
+
+        if not hasattr(module, '_non_persistent_buffers_set'):
+            module._non_persistent_buffers_set = set()
+
+        if not hasattr(module, 'input_spec'):
+            module.input_spec = None
+            if module.input_shape is not None:
+                module.input_spec = TensorSpec(shape=module.input_shape)
+
+        if not hasattr(module, 'batch_index'):
+            setattr(module, 'batch_index', 0)
+        if not hasattr(module, 'filter_index'):
+            setattr(module, 'filter_index', -1)
+        if not hasattr(module, 'in_sequence'):
+            setattr(module, 'in_sequence', False)
+
+        if not hasattr(module, 'in_sequence'):
+            if 'lstm' in class_name.lower() or 'gru' in class_name.lower() or 'rnn' in class_name.lower():
+                module.in_sequence = True
+            else:
+                module.in_sequence = False
+
+        if 'Conv' in class_name and 'Block' in class_name:
+            if not hasattr(module, 'sequence_rank'):
+                module.sequence_rank = 'cna'
+
+        if 'Conv' in class_name:
+            if not hasattr(module, 'depth_multiplier'):
+                if 'Depthwise' in class_name or 'Separable' in class_name:
+                    module.depth_multiplier = 1
+                else:
+                    module.depth_multiplier = None
+            if not hasattr(module, 'use_spectral'):
+                module.use_spectral = False
+
+    if not hasattr(layer, 'signature') or not hasattr(layer, '_signature'):
+        layer._signature = Signature()
+        if layer._input_shape is not None:
+            if is_tensor(layer._input_shape):
+                layer._signature.inputs["input"] = TensorSpec(shape=layer._input_shape, name="input")
+            elif isinstance(layer._input_shape, tuple) and isinstance(layer._input_shape[0], int):
+                layer._signature.inputs["input"] = TensorSpec(shape=to_tensor(layer._input_shape).to('int'), name="input")
+            elif isinstance(layer._input_shape, tuple):
+                for i in range(len(layer._input_shape)):
+                    layer._signature.inputs["input_{0}".format(i)] = TensorSpec(shape=layer._input_shape[i], name="input_{0}".format(i))
+        if layer._output_shape is not None:
+            if is_tensor(layer._output_shape):
+                layer._signature.outputs["output"] = TensorSpec(shape=layer._output_shape, name="output")
+            elif isinstance(layer._output_shape, tuple) and isinstance(layer._output_shape[0], int):
+                layer._signature.outputs["output"] = TensorSpec(shape=to_tensor(layer._output_shape).to('int'), name="output")
+            elif isinstance(layer._output_shape, tuple):
+                for i in range(len(layer._output_shape)):
+                    layer._signature.outputs["output_{0}".format(i)] = TensorSpec(shape=to_tensor(layer._output_shape[i]), name="output_{0}".format(i))
+        layer.signature = layer._signature
+
+    return layer
 
 
