@@ -70,7 +70,7 @@ class MixupCallback(RegularizationCallbacksBase):
         batch_size = int_shape(x)[0]
         index = arange(batch_size)
         index=cast(shuffle(index),'long')
-
+        this_loss=None
         mixed_x=None
         if get_backend()=='pytorch':
             mixed_x = lam * x + (1 - lam) * x[index, :]
@@ -86,18 +86,16 @@ class MixupCallback(RegularizationCallbacksBase):
 
             this_loss = lam * self.loss_criterion(pred, y_a) + (1 - lam) * self.loss_criterion(pred,y_b)
 
+        training_context['current_loss'] = training_context['current_loss'] + this_loss *self.loss_weight
+        if training_context['is_collect_data']:
+            training_context['losses'].collect('mixup_loss', training_context['steps'], float(to_numpy(this_loss * self.loss_weight)))
+
         if training_context['current_batch']==0:
             for item in mixed_x:
                 item=unnormalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])(to_numpy(item))
                 item=unnormalize(0, 255)(item)
                 array2image(item).save('Results/mixup_{0}.jpg'.format(get_time_suffix()))
 
-        if 'mixup_loss' not in training_context['losses']:
-            training_context['losses']['mixup_loss'] = []
-        training_context['current_loss'] = training_context['current_loss'] + this_loss *self.loss_weight
-        if training_context['is_collect_data']:
-            training_context['losses']['mixup_loss'].append(float(to_numpy(this_loss) * self.loss_weight))
-
 class CutMixCallback(RegularizationCallbacksBase):
     """Implementation. of the cutmix regularization
     CutMix is a way to combine two images. It comes from MixUp and Cutout. In this
@@ -202,11 +200,10 @@ class CutMixCallback(RegularizationCallbacksBase):
 
 
 
-        if 'cutmix_loss' not in training_context['losses']:
-            training_context['losses']['cutmix_loss'] = []
         training_context['current_loss'] = training_context['current_loss'] + this_loss *self.loss_weight
         if training_context['is_collect_data']:
-            training_context['losses']['cutmix_loss'].append(float(to_numpy(this_loss) * self.loss_weight))
+            training_context['losses'].collect('cutmix_loss', training_context['steps'], float(to_numpy(this_loss * self.loss_weight)))
+
 
 
 
