@@ -81,7 +81,11 @@ class PreprocessPolicyItem(object):
         self.count_false = 0
 
     def __call__(self, img):
-        bool_if =self.condition_if(img)
+        bool_if=None
+        if isinstance(self.condition_if,bool):
+            bool_if=self.condition_if
+        elif inspect.isfunction(condition_if) or callable(condition_if) :
+            bool_if =self.condition_if(img)
 
         if bool_if==True:
             if isinstance(self.then_process, list):
