@@ -1,7 +1,7 @@
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
-
+import copy
 import inspect
 import itertools
 import locale
@@ -143,20 +143,19 @@ class BatchSampler(Sampler):
             try:
                 _return_data = self.data_source[idx]
                 # filter sample
-                if self.sample_filter is None or self.sample_filter(_return_data):
-                    if not isinstance(_return_data, tuple):
-                        _return_data = _return_data,
-                    batch_data.append(_return_data)
+                if self.sample_filter is None or self.sample_filter(_return_data.value_list):
+                    batch_data.append(_return_data.value_list)
 
             except Exception as e:
                 print(e)
 
             if len(batch_data) == self.batch_size:
-                returnData = list(zip(*batch_data))
-                for i in range(len(returnData)):
-                    returnData[i] = np.asarray(list(returnData[i]))
+                returnData = copy.deepcopy(self.data_source.data_template)
+                batch_data = list(zip(*batch_data))
+                for i in range(len(batch_data)):
+                    returnData[returnData.key_list[i]] = np.asarray(list(batch_data[i]))
 
-                yield tuple(returnData)
+                yield returnData
                 batch_data = []
 
 
