@@ -1,6 +1,8 @@
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
+import os
+os.environ['TRIDENT_BACKEND'] = 'tensorflow'
 import builtins
 from typing import Optional,List,Tuple
 import collections
@@ -51,7 +53,7 @@ _pair = _ntuple(2)
 _triple = _ntuple(3)
 _quadruple = _ntuple(4)
 
-__all__ = ['Dense','Embedding' ,'Flatten', 'Concatenate', 'Concate', 'Add', 'Subtract', 'Conv1d', 'Conv2d', 'Conv3d', 'TransConv1d',
+__all__ = ['Dense','Embedding' ,'Flatten', 'Concatenate', 'Concate', 'Add', 'Subtract','Scale', 'Conv1d', 'Conv2d', 'Conv3d', 'TransConv1d',
            'TransConv2d', 'TransConv3d', 'DepthwiseConv1d', 'DepthwiseConv2d', 'DepthwiseConv3d', 'SeparableConv2d',
            'GatedConv2d','Upsampling2d', 'Reshape', 'Dropout', 'Lambda', 'SoftMax', 'Noise']
 
@@ -368,6 +370,91 @@ class SoftMax(Layer):
         return x
 
 
+
+class Scale(Layer):
+    """The Scale layer implements a per-tensor, per-channel, or per-element affine transformation and/or exponentiation by constant values.
+         Examples:
+                >>> x = to_tensor(ones((2,2,2,4)))
+                >>> layer1=Scale(scale=2,shift=0.5,power=1,mode='uniform')
+                >>> output1 = layer1(x)
+                >>> tf.reduce_all(output1==(x*2+0.5)**1)
+                <tf.Tensor: shape=(), dtype=bool, numpy=True>
+                >>> x = to_tensor(ones((2,2,2,4)))
+                >>> layer2=Scale(scale=to_tensor([1,2,3,4]),shift=0.5,power=1.2,mode='channel')
+                >>> output2 = layer2(x)
+                >>> tf.reduce_all(output2==pow((x*(to_tensor([1,2,3,4]).reshape((1,1,1,4)))+0.5),1.2))
+                <tf.Tensor: shape=(), dtype=bool, numpy=True>
+    """
+    def __init__(self, scale:(float,Tensor)=1.0, shift:(float,Tensor)=0.0, power:(float,Tensor)=1.0,mode='uniform',keep_output: bool=False,name:Optional[str] = None):
+        super(Scale, self).__init__(keep_output=keep_output,name=name)
+        self._scale=to_tensor(scale)
+        self._shift=to_tensor(shift)
+        self._power=to_tensor(power)
+
+        if mode == 'uniform' and (numel(self._scale)!=1 or numel(self._shift)!=1or numel(self._power)!=1):
+            raise ValueError('Scale/ Shift/ Power should float, 0d Tensor or One element Tensor whem mode=uniform')
+        if mode in [ 'uniform', 'channel', 'elementwise']:
+            self.mode = mode
+        else :
+            raise ValueError('Only [uniform,channel,elementwise] is valid value for mode ')
+
+    def build(self, input_shape):
+        def remove_from(name:str,*dicts):
+            for d in dicts:
+                if name in d:
+                    del d[name]
+        if self._built == False:
+            if self.mode =='uniform':
+                self.scale=tf.Variable(self._scale.copy(),trainable=True)
+                self.shift = tf.Variable(self._shift.copy(), trainable=True)
+                self.power = tf.Variable(self._power.copy(), trainable=True)
+            elif self.mode =='channel':
+                new_shape = [1] * (len(input_shape) + 1)
+                new_shape[self.filter_index] = self.input_filters
+                if ndim(self._scale) == 1 and numel(self._scale) in [1, self.input_filters]:
+                    if numel(self._scale) == 1:
+                        self._scale = repeat_elements(self._scale, self.input_filters, 0)
+                    self._scale = reshape(self._scale, new_shape)
+                if ndim(self._shift) == 1 and numel(self._shift) in [1, self.input_filters]:
+                    if numel(self._shift) == 1:
+                        self._shift = repeat_elements(self._shift, self.input_filters, 0)
+                    self._shift = reshape(self._shift, new_shape)
+                if ndim(self._power) == 1 and numel(self._power) in [1, self.input_filters]:
+                    if numel(self._power) == 1:
+                        self._power = repeat_elements(self._power, self.input_filters, 0)
+                    self._power = reshape(self._power, new_shape)
+
+                self.scale = tf.Variable(initial_value=self._scale.copy(), trainable=True)
+                self.shift = tf.Variable(initial_value=self._shift.copy(), trainable=True)
+                self.power = tf.Variable(initial_value=self._power.copy(), trainable=True)
+            elif self.mode == 'elementwise':
+                if ndim(self._scale) == 1 and numel(self._scale) ==1:
+                    self._scale=ones(input_shape)*self._scale
+                if ndim(self._shift) == 1 and numel(self._shift) ==1:
+                    self._shift=ones(input_shape)*self._shift
+                if ndim(self._power) == 1 and numel(self._power) ==1:
+                    self._power=ones(input_shape)*self._power
+
+                if int_shape(self._scale) ==input_shape:
+                    self._scale=self._scale.expand_dims(0,1)
+                if int_shape(self._shift) ==input_shape:
+                    self._shift=self._shift.expand_dims(0,1)
+                if int_shape(self._power) ==input_shape:
+                    self._power=self._power.expand_dims(0,1)
+                self.scale = tf.Variable(self._scale.copy(), trainable=True)
+                self.shift = tf.Variable(self._shift.copy(), trainable=True)
+                self.power = tf.Variable(self._power.copy(), trainable=True)
+            remove_from('_scale',self.__dict__, self._buffers)
+            remove_from('_shift',self.__dict__, self._buffers)
+            remove_from('_power',self.__dict__, self._buffers)
+            self._built = True
+
+    def forward(self, *x) -> Tensor:
+        x=enforce_singleton(x)
+        x = pow(x*self.scale+self.shift,self.power)
+        return x
+
+
 # def get_static_padding(rank,input_shape,kernal_shape,strides,dilations):
 #     if isinstance(strides,int):
 #         strides= _ntuple(rank)(strides)
@@ -481,7 +568,7 @@ class _ConvNd(Layer):
     def build(self, input_shape):
         if self._built == False:
             with self.name_scope:
-                self.input_filters = input_shape.as_list()[-1]
+                self.input_filters = to_numpy(input_shape)[self.filter_index]
                 if self.depthwise or self.separable:
                     if self.depth_multiplier is None:
                         self.depth_multiplier = 1
@@ -502,7 +589,7 @@ class _ConvNd(Layer):
                 if self.auto_pad:
                     self.padding = get_static_padding(self.rank, self.kernel_size ,
                                                        self.strides ,  self.dilation ,
-                                                       tuple(input_shape.as_list()[:-1]))
+                                                       tuple(to_list(input_shape)[:-1]))
                 else:
                     self.padding = normalize_padding(self.padding, self.rank)
 
@@ -532,7 +619,7 @@ class _ConvNd(Layer):
                         self.pointwise = tf.Variable(tf.random.normal( shape=[*pointwise_kernel_size, int(self.input_filters * channel_multiplier),int(self.num_filters)], mean=0, stddev=1) * 0.02, trainable=True, name='weight')
 
                 if self.use_bias:
-                    self.bias = tf.Variable(tf.zeros([int(self.num_filters)]), name='bias')
+                    self.bias = tf.Variable(tf.random.normal([int(self.num_filters)]), name='bias')
 
             self._built = True
 
@@ -550,7 +637,7 @@ class _ConvNd(Layer):
         if self.groups != 1:
             s += ', groups={groups}'
         if hasattr(self,'_input_shape') and self._input_shape is not None:
-            s += ', input_shape={0}, input_filters={1}'.format(self._input_shape.as_list(), self.input_filters)
+            s += ', input_shape={0}, input_filters={1}'.format(self._input_shape, self.input_filters)
         if hasattr(self,'_output_shape') and self._output_shape is not None:
             s += ', output_shape={0}'.format(
                 self._output_shape if isinstance(self._output_shape, (list, tuple)) else self._output_shape.as_list())
@@ -1365,7 +1452,7 @@ class SingleImageLayer(Layer):
     def build(self, input_shape):
         if self._built == False:
             self.weight = tf.Variable(to_tensor(self.origin_image.clone()), name='weight')
-            self.input_filters = input_shape.get_shape().as_list()[-1]
+            self.input_filters = to_numpy(input_shape)[self.filter_index]
             self._built = True
 
     def forward(self, x):
