@@ -21,7 +21,7 @@ from torch._six import container_abcs
 from torch.nn import init
 
 from trident.backend.common import *
-from trident.backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential, summary
+from trident.backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential, summary, fix_layer
 from trident.data.image_common import *
 from trident.data.utils import download_model_from_google_drive
 from trident.layers.pytorch_activations import get_activation, Identity, Relu, PRelu
@@ -123,6 +123,7 @@ def SEResNet50_IR_512(
     if pretrained == True:
         download_model_from_google_drive('1aLYbFvtvsV2gQ16D_vwzrKdbCgij7IoZ', dirname, 'arcface_se_50_512.pth')
         recovery_model = torch.load(os.path.join(dirname, 'arcface_se_50_512.pth'))
+        recovery_model = fix_layer(recovery_model)
         recovery_model.name = 'arcface_se_50_512'
         recovery_model.eval()
         recovery_model.to(_device)
