@@ -21,7 +21,7 @@ from torch.nn import init
 from torch.nn.parameter import Parameter
 
 from trident.backend.common import *
-from trident.backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential
+from trident.backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential, fix_layer
 from trident.data.image_common import *
 from trident.data.utils import download_model_from_google_drive
 from trident.layers.pytorch_activations import get_activation, Identity
@@ -136,19 +136,19 @@ def MobileNetV2(include_top=True,
     if pretrained==True:
         download_model_from_google_drive('1ULenXTjOO5PdT3fHv6N8bPXEfoJAn5yL',dirname,'mobilenet_v2.pth')
         recovery_model=torch.load(os.path.join(dirname,'mobilenet_v2.pth'))
+        recovery_model = fix_layer(recovery_model)
         recovery_model.eval()
         recovery_model.to(_device)
         if include_top==False:
-            recovery_model.__delitem__(-1)
-            recovery_model.__delitem__(-1)
-            recovery_model.__delitem__(-1)
-            recovery_model.__delitem__(-1)
+            recovery_model.remove_at(-1)
+            recovery_model.remove_at(-1)
+            recovery_model.remove_at(-1)
+            recovery_model.remove_at(-1)
             mob.class_names = []
         else:
             if classes!=1000:
-                new_fc = Dense(classes, activation=None, name='fc')
-                new_fc.input_shape=recovery_model.fc.input_shape
-                recovery_model.fc=new_fc
+                recovery_model.remove_at(-1)
+                recovery_model.add_module('fc', Dense(classes, activation=None, name='fc'))
                 mob.class_names=[]
         mob.model=recovery_model
     return mob
\ No newline at end of file
