@@ -22,7 +22,7 @@ from torch.nn.parameter import Parameter
 
 from trident.backend.common import *
 from trident.backend.model import *
-from trident.backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential
+from trident.backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential, fix_layer
 from trident.data.image_common import *
 from trident.data.utils import download_model_from_google_drive
 from trident.layers.pytorch_activations import get_activation, Identity, Relu
@@ -118,15 +118,17 @@ def VGG11(include_top=True,
     if pretrained==True:
         download_model_from_google_drive('1PV9-AwgD1v-JxDRzduOjjGduIR7MDhPW',dirname,'vgg11.pth')
         recovery_model=torch.load(os.path.join(dirname,'vgg11.pth'))
+        recovery_model = fix_layer(recovery_model)
         recovery_model.name='vgg11'
         recovery_model.eval()
         recovery_model.to(_device)
         if include_top==False:
-            [recovery_model.__delitem__(-1) for i in range(7)]
+            [recovery_model.remove_at(-1) for i in range(7)]
             vgg11.class_names = []
         else:
             if classes!=1000:
-                recovery_model.fc3=Dense(classes,use_bias=True,activation='softmax')
+                recovery_model.remove_at(-1)
+                recovery_model.add_module('fc3', Dense(classes,use_bias=True,activation='softmax'))
                 vgg11.class_names = []
         vgg11.model=recovery_model
     return vgg11
@@ -150,15 +152,17 @@ def VGG13(include_top=True,
     if pretrained==True:
         download_model_from_google_drive('1wx67gmQ8eHWXs2mhJmNl-t-cFNw7dJ7O',dirname,'vgg13.pth')
         recovery_model=torch.load(os.path.join(dirname,'vgg13.pth'))
+        recovery_model = fix_layer(recovery_model)
         recovery_model.name = 'vgg13'
         recovery_model.eval()
         recovery_model.to(_device)
         if include_top==False:
-            [recovery_model.__delitem__(-1) for i in range(7)]
+            [recovery_model.remove_at(-1) for i in range(7)]
             vgg13.class_names = []
         else:
             if classes!=1000:
-                recovery_model.fc3=Dense(classes,use_bias=True,activation='softmax')
+                recovery_model.remove_at(-1)
+                recovery_model.add_module('fc3', Dense(classes, use_bias=True, activation='softmax'))
                 vgg13.class_names = []
         vgg13.model=recovery_model
     return vgg13
@@ -179,15 +183,17 @@ def VGG16(include_top=True,
     if pretrained==True:
         download_model_from_google_drive('1uXiH5MSy1rvxrHjW4uB9E2BHMM8b0Fwr',dirname,'vgg16.pth')
         recovery_model=torch.load(os.path.join(dirname,'vgg16.pth'))
+        recovery_model = fix_layer(recovery_model)
         recovery_model.name = 'vgg16'
         recovery_model.eval()
 
         if include_top==False:
-            [recovery_model.__delitem__(-1) for i in range(7)]
+            [recovery_model.remove_at(-1) for i in range(7)]
             vgg16.class_names = []
         else:
             if classes!=1000:
-                recovery_model.fc3=Dense(classes,use_bias=True,activation='softmax')
+                recovery_model.remove_at(-1)
+                recovery_model.add_module('fc3', Dense(classes, use_bias=True, activation='softmax'))
                 vgg16.class_names = []
         recovery_model.to(_device)
         vgg16.model=recovery_model
@@ -208,15 +214,17 @@ def VGG19(include_top=True,
     if pretrained==True:
         download_model_from_google_drive('1nqQJLYMzeiUX9hji39-rrBUG42YyjhYg',dirname,'vgg19.pth')
         recovery_model=torch.load(os.path.join(dirname,'vgg19.pth'))
+        recovery_model = fix_layer(recovery_model)
         recovery_model.name = 'vgg19'
         recovery_model.eval()
 
         if include_top==False:
-            [recovery_model.__delitem__(-1) for i in range(7)]
+            [recovery_model.remove_at(-1) for i in range(7)]
             vgg19.class_names = []
         else:
             if classes!=1000:
-                recovery_model.fc3=Dense(classes,use_bias=True,activation='softmax')
+                recovery_model.remove_at(-1)
+                recovery_model.add_module('fc3', Dense(classes, use_bias=True, activation='softmax'))
                 vgg19.class_names = []
         recovery_model.to(_device)
         vgg19.model=recovery_model
