@@ -54,6 +54,7 @@ class Optimizer(optimizer.Optimizer):
         self._base_lr = 1e-3
         self.gradient_centralization=None
 
+
     def adjust_learning_rate(self, new_lr, verbose=True):
         """
 
@@ -129,7 +130,7 @@ class Adam(optim.Adam):
     """
 
     def __init__(self, params, lr=1e-3, betas=(0.9, 0.999), eps=1e-8, weight_decay=0, amsgrad=False,
-                 gradient_centralization=None):
+                 gradient_centralization=None,**kwargs):
         super().__init__(params, lr=lr, betas=betas, eps=eps, weight_decay=weight_decay, amsgrad=amsgrad)
 
     def adjust_learning_rate(self, new_lr, verbose=True):
