@@ -49,8 +49,7 @@ def total_variation_norm_reg(output:torch.Tensor,reg_weight=1):
     diff_i = torch.sum(torch.pow(output[:, :, :, 1:] - output[:, :, :, :-1], 2))
     diff_j = torch.sum(torch.pow(output[:, :, 1:, :] - output[:, :, :-1, :], 2))
     tv_loss = true_divide(0.5*(true_divide(diff_i,w) + true_divide(diff_j,h)),b)
-    loss = reg_weight * tv_loss
-    return loss
+    return reg_weight * tv_loss
 
 
 def get_reg(reg_name):
