@@ -1,7 +1,7 @@
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
-
+import numbers
 import copy
 import gc
 import inspect
@@ -19,8 +19,13 @@ import numpy as np
 import torch
 import torch.nn as nn
 
+from trident.data.dataset import Iterator,NumpyDataset,LabelDataset
+from trident.optims.trainers import TrainingPlan
+from trident.data.data_provider import DataProvider
+
 from trident import __version__
 from trident.backend.common import *
+from trident.backend.tensorspec import *
 from trident.backend.pytorch_backend import *
 from trident.backend.pytorch_ops import *
 from trident.backend.model import ModelBase, progress_bar
@@ -41,6 +46,7 @@ from trident.misc.visualization_utils import tile_rgb_images, loss_metric_curve
 
 __all__ = [ 'Model', 'ImageClassificationModel', 'ImageDetectionModel', 'ImageGenerationModel',
            'ImageSegmentationModel','FaceRecognitionModel']
+_session = get_session()
 
 _, term_width = get_terminal_size()
 term_width = int(term_width)
@@ -87,36 +93,53 @@ class Model(ModelBase):
     def _initial_graph(self, inputs=None, input_shape=None,output=None,initializer=None):
         if output is None:
             raise ValueError('There is at least one output')
-        if isinstance(output,(np.ndarray,torch.Tensor)) and input_shape is None:
-            input_shape=output.shape
+
+        if isinstance(output,(np.ndarray,Tensor)) and input_shape is None:
+            input_shape=squeeze(output.shape)
 
         if inputs is None:
-
             if input_shape is None:
                 raise ValueError('You should assign inputs or input shape')
-            else:
-                input_shape = to_list(input_shape)
+            elif isinstance(input_shape,TensorSpec):
+                self.inputs[input_shape.name]=input_shape
+            elif isinstance(input_shape, dict):
+                for k, v in input_shape.items():
+                    if is_tensor(v):
+                        self.inputs[k] = TensorSpec(shape=v, name=k)
+                    elif isinstance(v,TensorSpec):
+                        self.inputs[v.name] = v
+            elif isinstance(input_shape, (tuple,list)) and all([isinstance(item,int) for item in input_shape]):
                 input_name = 'input'
-                self.inputs[input_name] = input_shape
+                self.inputs[input_name] = TensorSpec(shape=to_tensor(input_shape).int(), name=input_name)
+            else:
+                input_shape=unpack_singleton(input_shape)
+                if is_tensor(input_shape):
+                    input_name = 'input'
+                    self.inputs[input_name] =TensorSpec(shape=input_shape,name=input_name)
+                else:
+                    for m in range(len(input_shape)):
+                        self.inputs[ 'input_{0}'.format(m)] = TensorSpec(shape=(input_shape[m]), name= 'input_{0}'.format(m))
         elif isinstance(inputs, (tuple, list)):
             if len(inputs)==1 and is_tensor(inputs[0]):
                 input_name = 'input'
-                self.inputs[input_name] = int_shape(inputs[0])
+                self.inputs[input_name] = TensorSpec(shape=to_tensor(int_shape(inputs[0])[self.batch_index+1:]),name=input_name)
             else:
                 for m in range(len(inputs)):
                     inp=inputs[m]
                     if is_tensor(inp) or isinstance(inp,np.ndarray):
                         input_name = 'input_{0}'.format(m)
-                        self.inputs[input_name] = list(int_shape(inp))[self.batch_index+1:]
+                        self.inputs[input_name] = TensorSpec(shape=to_tensor(int_shape(inputs[m])[self.batch_index+1:]),name=input_name)
         elif isinstance(inputs, dict):
             for k, v in inputs.items():
-                if is_tensor(v)or isinstance(v,np.ndarray):
-                    self.inputs[k] =  list(int_shape(v))[self.batch_index+1:]
+                if isinstance(v, TensorSpec):
+                    self.inputs[k] =v
+                elif is_tensor(v)or isinstance(v,np.ndarray):
+                    self.inputs[k] =   TensorSpec(shape=to_tensor(int_shape(v)[self.batch_index+1:]),name=k)
         elif is_tensor(inputs):
-            self.inputs['input'] = list(int_shape(inputs))[self.batch_index+1:]
+            self.inputs['input'] =   TensorSpec(shape=to_tensor(int_shape(inputs)[self.batch_index+1:]),name='input')
         elif isinstance(inputs,np.ndarray):
             inputs=to_tensor(inputs)
-            self.inputs['input'] = list(int_shape(inputs))[self.batch_index+1:]
+            self.inputs['input'] =   TensorSpec(shape=to_tensor(int_shape(inputs)[self.batch_index+1:]),name='input')
 
         #single model
         if isinstance(output, (Layer, nn.Module)):
@@ -128,44 +151,42 @@ class Model(ModelBase):
 
             # output.cpu()
             if  output.built and hasattr(output,'_output_shape') and  output._output_shape is not None:
-                self._model = output
-                self._outputs['output'] = to_list(output._output_shape)
-                self._targets['target'] = to_list(output._output_shape)
-
-                self._model.signature = get_signature(self._model.forward, 'model')
-                self._model.signature.inputs = copy.deepcopy(self.inputs)
-                self._model.signature.outputs = copy.deepcopy(self._outputs)
+                self._model = fix_layer(output)
+                self._model.signature = None
+                if self._model.signature is not None and hasattr(self._model.signature,"outputs"):
+                    self._outputs['output'] = self._model.signature.outputs
+                    self._targets = OrderedDict()
+                    for k,v in self._model.signature.outputs.item_list:
+                        self._targets[k.replace("output","target")]=v
                 self._signature = self._model.signature
             else:
                 out=None
                 if inputs is not None:
                     args=None
                     if isinstance(inputs,dict):
-                        out = output(*list(inputs.values()))
+                        out = fix_layer(output)(*list(inputs.values()))
                     elif isinstance(inputs,(list,tuple)):
-                        out = output(*inputs)
+                        out = fix_layer(output)(*inputs)
                     else:
-                        out = output(inputs)
+                        out = fix_layer(output)(inputs)
 
                 else:
 
-                    output.input_shape = input_shape
+                    output.input_shape =to_tensor( input_shape)
                     dummay_input = to_tensor(np.random.standard_normal((1,)+tuple(input_shape)).astype(np.float32)).to(get_device())
                     # prevent pytorch 'ValueError: Expected more than 1 value per channel when training, got input size ....
                     output.to(get_device())
                     output.eval()
-                    out = output(dummay_input)
+                    out = fix_layer(output)(dummay_input)
+                output.signature = None
                 self._model = output
                 if isinstance(out, torch.Tensor):
-                    self._outputs['output'] = to_list(out.size())[self.batch_index+1:]
-                    self._targets['target'] = to_list(out.size())[self.batch_index+1:]
+                    self._outputs['output'] = TensorSpec(shape=to_tensor(int_shape(out)[self.batch_index+1:]),name='output')
+                    self._targets['target'] = TensorSpec(shape=to_tensor(int_shape(out)[self.batch_index+1:]),name='target')
                 else:
                     for i in range(len(out)):
-                        self._outputs['output_{0}'.format(i)] = to_list(out[i].size())[self.batch_index+1:]
-                        self._targets['target_{0}'.format(i)] = to_list(out[i].size())[self.batch_index+1:]
-            self._model.signature = get_signature(self._model.forward,'model')
-            self._model.signature.inputs = copy.deepcopy(self.inputs)
-            self._model.signature.outputs=copy.deepcopy(self._outputs)
+                        self._outputs['output_{0}'.format(i)] = TensorSpec(shape=to_tensor(int_shape(out[i])[self.batch_index+1:]),name='output_{0}'.format(i))
+                        self._targets['target_{0}'.format(i)] = TensorSpec(shape=to_tensor(int_shape(out[i])[self.batch_index+1:]),name='target_{0}'.format(i))
             self._signature=self._model.signature
         elif isinstance(output, (list,tuple)) and  all([isinstance(m,(nn.Module)) for m in output]):
             output_list = []
@@ -177,40 +198,37 @@ class Model(ModelBase):
                     op.to(get_device())
                     # prevent pytorch 'ValueError: Expected more than 1 value per channel when training, got input size ....
                     op.eval()
-                    out=op(dummay_input)
+                    out=fix_layer(op)(dummay_input)
                     model_list.append(op)
                     output_list.extend(*out)
             model = Combine(model_list)
             self._model = model
             self.name = model.name
             for i in range(len(output_list)):
-                self._outputs['output_{0}'.format(i)] = list(int_shape(output_list[i]))[self.batch_index+1:]
-                self._targets['target_{0}'.format(i)] = list(int_shape(output_list[i]))[self.batch_index+1:]
-            self._model.signature = get_signature(self._model.forward,'model')
-            self._model.signature.inputs = copy.deepcopy(self.inputs)
-            self._model.signature.outputs = copy.deepcopy(self._outputs)
+                self._outputs['output_{0}'.format(i)] = TensorSpec(shape=to_tensor(int_shape(output_list[i])[self.batch_index + 1:]), name='output_{0}'.format(i))
+                self._targets['target_{0}'.format(i)] = TensorSpec(shape=to_tensor(int_shape(output_list[i])[self.batch_index + 1:]), name='target_{0}'.format(i))
             self._signature = self._model.signature
         elif isinstance(output,(np.ndarray,torch.Tensor)):
             #style transfer , or adversarial attack
             self._model =to_tensor(output,requires_grad=True)
             out=self._model
-            self._outputs['output'] = to_list(out.size())[self.batch_index + 1:]
-            self._targets['target'] = to_list(out.size())[self.batch_index + 1:]
-            self._model.signature =Signature()
-            self._model.signature.outputs = copy.deepcopy(self._outputs)
-            self._signature = self._model.signature
+            self._outputs['output'] = TensorSpec(shape=to_tensor(int_shape(out)[self.batch_index + 1:]), name='output')
+            self._targets['target'] = TensorSpec(shape=to_tensor(int_shape(out)[self.batch_index + 1:]), name='target')
+
         else:
             raise ValueError('Invalid output')
 
 
         self.training_context['current_model'] = self._model
-        save_path = os.path.join('Models', '{0}.pth.tar_'.format(self._model.name))
-        self.save_path =sanitize_path(make_dir_if_need(save_path))
+        if hasattr(self._model,'name') and 'name' in self.__dict__:
+            delattr(self,'name')
+        if self.save_path is None:
+            save_path = os.path.join('Models', '{0}.pth.tar_'.format(self._model._name))
+            self.save_path =sanitize_path(make_dir_if_need(save_path))
+        else:
+            self.save_path = sanitize_path(make_dir_if_need(self.save_path))
         self.training_context['save_path'] = self.save_path
-        if isinstance(self._model,Layer):
-            for module in self._model.modules():
-                if not hasattr(module, '_non_persistent_buffers_set'):
-                    module._non_persistent_buffers_set = set()
+
 
     @property
     def outputs(self):
@@ -239,21 +257,104 @@ class Model(ModelBase):
 
     @property
     def layers(self):
-        if self._model is not None and  isinstance(self._model, torch.Tensor):
-            return None
+        if self._model is not None and isinstance(self._model, Layer):
+            return self._model.nodes
         else:
-            return self._model._nodes
+            return []
 
-    def complie(self, optimizer, losses=None, metrics=None, loss_weights=None, sample_weight_mode=None,
-                weighted_metrics=None, target_tensors=None):
-        self.with_optimizer(optimizer)
-        if losses is not None and isinstance(losses, (list, tuple)):
-            for loss in losses:
-                self.with_loss(loss)
-        if metrics is not None and isinstance(metrics, (list, tuple)):
-            for metric in metrics:
-                self.with_metric(metric)
+    def complie(self,optimizer="Adam",
+                    loss=None,
+                    metrics=None,
+                    loss_weights=None,
+                    **kwargs
+                 ):
+        self.with_optimizer(optimizer,lr=2e-3,betas=(0.9, 0.999))
+        if loss is not None :
+            if isinstance(loss,str) or callable(loss) or inspect.isfunction(loss) or inspect.isclass(loss):
+                loss_weights=1.0 if loss_weights is None or not isinstance(loss,numbers.Number) else loss_weights
+                self.with_loss(loss,loss_weight=loss_weights)
+            elif isinstance(loss, list):
+                if loss_weights is not None and isinstance(loss_weights, list) and len(loss_weights)==len(loss):
+                    for k in range(len(loss)):
+                        loss_item=loss[k]
+                        weight = loss_weights[k] if isinstance(loss_weights[k], numbers.Number) else 1.0
+                        self.with_loss(loss_item,loss_weight=weight)
+                else:
+                    for loss_item in loss:
+                        self.with_loss(loss_item)
 
+            elif isinstance(loss,dict):
+                if loss_weights is not None and isinstance(loss_weights, dict):
+                    for k,v in loss.items():
+                        if k in loss_weights:
+                            weight=loss_weights[k] if isinstance(loss_weights[k], numbers.Number) else 1.0
+                            self.with_loss(v, loss_weight=weight,name=k)
+                        else:
+                            self.with_loss(v, loss_weight=1.0,name=k)
+                else:
+                    for k, v in loss.items():
+                        self.with_loss(v, loss_weight=1.,name=k)
+        if metrics is not None:
+            if isinstance(metrics,str) or callable(metrics) or inspect.isfunction(metrics) or inspect.isclass(metrics):
+                self.with_metric(metrics)
+            elif isinstance(metrics, list):
+                for metric in metrics:
+                    self.with_metric(metric)
+            elif isinstance(metrics, dict):
+                for k, v in metrics.items():
+                    self.with_metric(v, name=k)
+        return self
+
+    def fit(self,
+            x=None,
+            y=None,
+            batch_size=8,
+            epochs=1,
+            verbose=1,
+            callbacks=None,
+            validation_split=0.0,
+            validation_data=None,
+            shuffle=True,
+            class_weight=None,
+            sample_weight=None,
+            initial_epoch=0,
+            steps_per_epoch=None,
+            validation_steps=None,
+            validation_batch_size=8,
+            validation_freq=1,
+            max_queue_size=10,
+            workers=1,
+            use_multiprocessing=False,
+            ):
+
+        split_range=list(range(len(x)))
+        random.shuffle(split_range)
+        cutoff=int(validation_split*len(x))
+        mask_validate=split_range[:cutoff]
+        mask_train = split_range[cutoff:]
+        train_x=x[mask_train]
+        train_y = y[mask_train]
+        validate_x = x[mask_validate]
+        validate_y = y[mask_validate]
+
+
+        data_ds=NumpyDataset(data=train_x,symbol="input")
+        label_ds = LabelDataset(labels=train_y, symbol="target")
+        dataprovider=DataProvider(traindata=Iterator(data=data_ds,label=label_ds,minibatch_size=batch_size,is_shuffe=shuffle,buffer_size=max_queue_size,workers=workers))
+        if validation_split>0:
+            data_test_ds = NumpyDataset(data=train_x, symbol="input")
+            label_test_ds = LabelDataset(labels=train_y, symbol="target")
+            dataprovider.testdata=Iterator(data=data_test_ds, label=label_test_ds, minibatch_size=validation_batch_size, is_shuffe=shuffle, buffer_size=max_queue_size, workers=workers)
+
+
+        plan = TrainingPlan() \
+            .add_training_item(self) \
+            .with_data_loader(dataprovider) \
+            .repeat_epochs(epochs) \
+            .within_minibatch_size(batch_size) \
+            .print_progress_scheduling(1, unit='epoch') \
+            .out_sample_evaluation_scheduling(validation_steps, unit='epoch') \
+            .save_model_scheduling(1, unit='epoch').start_now()
         return self
 
     def with_optimizer(self, optimizer, **kwargs):
@@ -267,8 +368,8 @@ class Model(ModelBase):
         #                                                                factor=0.5, patience=5, threshold=1e-4,
         #                                                                cooldown=0, min_lr=1e-10, eps=1e-8)
         self.base_lr = kwargs.get('lr', kwargs.get('learning_rate', 1e-3))
-        self.training_context['optimizer'] = self.optimizer
-        self.training_context['base_lr'] = self.base_lr
+        # self.training_context['optimizer'] = self.optimizer
+        # self.training_context['base_lr'] = self.base_lr
         self.training_context['current_lr'] = self.base_lr
         return self
 
@@ -285,6 +386,7 @@ class Model(ModelBase):
             else:
                 loss_class = get_loss(loss)
                 alias = loss if loss_class is not None else alias
+                #loss can add mutiple times.
                 if  alias in self._losses:
                     dup_keys=[key for  key in self._losses.key_list if alias+'_' in key]
                     alias = alias + '_' + str(len(dup_keys)+1)
@@ -293,7 +395,7 @@ class Model(ModelBase):
                     argnames = get_signature(self._losses[alias].forward,alias)
                 else:
                     argnames = get_signature(self._losses[alias].__call__,alias)
-        elif inspect.isclass(loss) and inspect._is_type(loss):
+        elif inspect.isclass(loss) and inspect._is_type(loss): #The loss is a class but not initialized yet.
             alias = loss.__class__.__name__ if alias is None or len(alias) == 0 else alias
             if alias in self._losses:
                 dup_keys = [key for key in self._losses.key_list if alias + '_' in key]
@@ -305,7 +407,7 @@ class Model(ModelBase):
             else:
                 argnames = get_signature(self._losses[alias].__call__,alias)
 
-        elif not inspect.isfunction(loss) and callable(loss):
+        elif not inspect.isfunction(loss) and callable(loss):#The loss is a class and initialized yet.
             alias = loss.__class__.__name__ if alias is  None or len(alias) == 0 else alias
             if alias in self._losses:
                 dup_keys = [key for key in self._losses.key_list if alias + '_' in key]
@@ -333,8 +435,7 @@ class Model(ModelBase):
             try:
                 self._losses[alias].signature=argnames
                 self._losses[alias].signature.name = alias
-                if (len(self._losses[alias].signature.outputs) == 1 and self._losses[alias].signature.outputs.value_list[0] is None) or len(
-                        self._losses[alias].signature.outputs) == 0:
+                if (len(self._losses[alias].signature.outputs) == 1 and self._losses[alias].signature.outputs.value_list[0] is None) or len(self._losses[alias].signature.outputs) == 0:
                     self._losses[alias].signature.outputs = OrderedDict()
                     self._losses[alias].signature.outputs[alias] = None
                 if hasattr(self._losses[alias], 'is_logsoftmax'):
@@ -363,15 +464,16 @@ class Model(ModelBase):
 
         if isinstance(metric, str):
             alias = metric if len(alias) == 0 else alias
-            metric_class = get_metric(metric)
+            metric_fn = get_metric(metric)
             if alias in self._metrics:
                 dup_keys = [key for key in self._metrics.key_list if alias + '_' in key]
                 alias = alias + '_' + str(len(dup_keys) + 1)
-            self._metrics[alias] = metric_class(**kwargs)  if len(kwargs)>0 else metric_class()
-            if hasattr(metric, 'forward'):
-                argnames = get_signature(self._metrics[alias].forward,alias)
+            spec = inspect.getfullargspec(metric_fn)
+            if len(spec.args) >= 2 and len(spec.args) - 0 if spec.defaults is None else len(spec.defaults) == 2:
+                self._metrics[alias] = metric
             else:
-                argnames = get_signature(self._metrics[alias].__call__,alias)
+                self._metrics[alias] = partial(metric_fn, **kwargs)
+            argnames = get_signature(metric_fn, alias)
         elif inspect.isclass(metric) and inspect._is_type(metric):
             alias = metric.__class__.__name__ if len(alias) == 0 else alias
             if alias in self._metrics:
@@ -428,7 +530,7 @@ class Model(ModelBase):
         elif reg is callable or inspect.isfunction(reg):
             reg_fn = reg
         args = get_signature(reg_fn)
-        if 'reg_weight' in args:
+        if 'reg_weight' in args.inputs.key_list:
             args.pop('reg_weight')
         self._regs[reg_fn.__name__] = partial(reg_fn, **kwargs)
         self._regs[reg_fn.__name__].signature = args
@@ -514,26 +616,6 @@ class Model(ModelBase):
         self.optimizer.param_groups[0]['lr'] = lr
         self.training_context['current_lr'] = lr
 
-    def rebinding_input_output(self, input_shape):
-        if self._model is not None and  isinstance(self._model, torch.Tensor):
-            pass
-        elif input_shape == self._model._input_shape:
-            pass
-        else:
-            if len(self.inputs) == 1:
-                input_shape_list = to_numpy(input_shape).tolist()
-                self._model.input_shape = input_shape_list
-                self.inputs[self.inputs.key_list[0]] = input_shape_list
-
-            dummay_input = to_tensor(np.random.standard_normal((2, *input_shape_list))).to(
-                torch.device("cuda" if self._model.weights[0].data.is_cuda else "cpu"))
-            out = self._model(dummay_input)
-            if isinstance(out, torch.Tensor) == len(self.targets) == 1:
-                self.targets[self.targets.key_list[0]] = list(out.size())[1:] if len(out.size()) > 1 else []
-                self._model.output_shape = out.size()[1:]
-            elif isinstance(out, tuple):
-                for k in range(len(out)):
-                    self.targets[self.targets.key_list[k]] = list(out[k].size())[1:] if len(out[k].size()) > 1 else []
 
     def do_on_training_start(self):
         self.train()
@@ -542,45 +624,61 @@ class Model(ModelBase):
         self.eval()
 
     def do_on_epoch_start(self):
-        # if self.training_context['current_epoch'] < self.warmup:
-        #     lr = 1e-5 * (self.training_context['current_epoch'] + 1)
-        #     self.optimizer.param_groups[0]['lr'] = lr
-        #     self.training_context['current_lr'] = lr
-        # elif self.training_context['current_epoch'] == self.warmup:
-        #     self.optimizer.param_groups[0]['lr'] = self.base_lr
-        #     self.training_context['current_lr'] =self.base_lr
+        self.training_context['time_epoch_start']=time.time()
+        if self.training_context['steps']==0:
+            self.training_context['time_epoch_progress']=self.training_context['time_epoch_start']
+        if self.warmup>0 and self.training_context['current_epoch'] < self.warmup:
+            lr = 1e-6 * (self.training_context['current_epoch'] + 1)
+            self.optimizer.adjust_learning_rate(self.base_lr, verbose=True)
+            self.training_context['current_lr'] = lr
+        elif self.warmup>0 and self.training_context['current_epoch'] == self.warmup:
+            self.optimizer.adjust_learning_rate(self.base_lr, verbose=True)
+            self.training_context['current_lr'] =self.base_lr
         if self.model.device == 'cuda':
             torch.cuda.synchronize()
             torch.cuda.empty_cache()
             gc.collect()
 
     def do_on_epoch_end(self):
-        pass  # if self.training_context['current_epoch'] > self.warmup:  #     if self.lr_scheduler is not None:  #         self.lr_scheduler.step(np.array(self.training_context['metrics'][list(self._metrics.keys())[0]]).mean())  #         self.training_context['current_lr'] = self.optimizer.lr  #     if self.optimizer.param_groups[0]['lr'] < 1e-8:  #         self.optimizer.param_groups[0]['lr'] = 0.05 * self.base_lr  #         self.training_context['current_lr'] =  0.05 * self.base_lr  # elif  self.warmup>0 and self.training_context['current_epoch'] == self.warmup:  #     self.optimizer.adjust_learning_rate(self.base_lr, True)  #     self.training_context['current_lr'] =self.base_lr  # elif   self.warmup>0 and self.training_context['current_epoch'] < self.warmup:  #     self.optimizer.adjust_learning_rate(1e-5*(self.training_context['current_epoch']+1), True)  #     self.training_context['current_lr'] = 1e-5*(self.training_context['current_epoch']+1)
+        self.training_context['time_epoch_end'] = time.time()
+
+
 
     def do_on_batch_start(self):
+        self.training_context['time_batch_start'] = time.time()
+        if self.training_context['steps']==0:
+            self.training_context['time_batch_progress']=self.training_context['time_batch_start']
         if self.model.device == 'cuda':
             torch.cuda.empty_cache()
 
 
+
     def do_on_batch_end(self):
+        self.training_context['time_batch_end'] = time.time()
+        self.training_context['steps'] += 1
+        if self.training_context['steps']%_session.epoch_equivalent==0:
+            if self.warmup>0 and self.warmup==self.training_context['steps']//_session.epoch_equivalent:
+                self.adjust_learning_rate(self.training_context['base_lr'])
+                self.warmup=0
         if self.training_context['current_batch'] == 0 and self.training_context['is_print_batch_progress']==True:
             temp = OrderedDict()
             for k in self.training_context['losses'].key_list:
                 if len(self.training_context['losses'][k])>0:
-                    temp[k] = self.training_context['losses'][k][-1]
+                    temp[k] = self.training_context['losses'][k][-1][-1]
             print(temp)
             if is_tensor(self._model):
                 if self._model.grad is not None:
                     self._model.requires_grad=False
                     self._model.requires_grad=True
-            # elif isinstance(self._model, nn.Module):
+            #elif isinstance(self._model, nn.Module):
             #     self._model.zero_grad()
 
     def do_on_data_received(self, train_data, test_data):
 
         # fields=train_data._fields
         # for i in range(len(fields)):
-
+        if train_data is None and test_data is None:
+            return self.training_context['train_data'] , self.training_context['test_data']
         if 'data_feed' not in self.training_context or len(self.training_context['data_feed']) == 0 or self.training_context['current_batch']+self.training_context['current_epoch']==0:
             try:
 
@@ -591,8 +689,8 @@ class Model(ModelBase):
                 if train_data is not None:
                     # check input
                     for arg in self._model.signature.inputs.key_list:
-                        if  arg in data_feed and arg in available_fields:
-                            available_fields.remove(arg)
+                        if  arg in data_feed and data_feed[arg] in available_fields:
+                            available_fields.remove(data_feed[arg])
                         else:
                             data_feed[arg] = ''
                             if len(train_data) == 1 and len(self._model.signature.inputs.key_list)==1:
@@ -658,8 +756,8 @@ class Model(ModelBase):
                                             Warning(
                                                 'target argment {0} cannot mapping to any data, please check it and update the datafeed'.format(
                                                     arg))
-                            if len(self.targets) == 1 and data_feed[self.targets.key_list[0]] != None:
-                                self.training_context['current_target'] = train_data[data_feed[self.targets.key_list[0]]]
+                            # if len(self.targets) == 1 and data_feed[self.targets.key_list[0]] != None:
+                            #     self.training_context['current_target'] = train_data[data_feed[self.targets.key_list[0]]]
 
                     # if len(self._signature.inputs.key_list) == 1 and data_feed[self._signature.inputs.key_list[0]] != None:
                     #     self.training_context['data_feed'] = data_feed
@@ -673,11 +771,12 @@ class Model(ModelBase):
         # convert to tensor
         try:
             data_feed = self.training_context['data_feed']
-            input_list = [data_feed[arg] for arg in self._signature.inputs.key_list]
+            input_list = [data_feed[arg] for arg in self._model._signature.inputs.key_list]
             for item in train_data.key_list:
+
                 if item in input_list:
                     # only model 's input argments
-                    train_data[item] = to_tensor(train_data[item].copy())#.cpu()
+                    train_data[item] = to_tensor(train_data[item].copy()) #.cpu()
                     if 'float' in str(train_data[item].dtype):
                         train_data[item].require_grads=True
                 elif item in self.targets.key_list or data_feed:
@@ -834,7 +933,7 @@ class Model(ModelBase):
                 'backend':'pytorch',
                 'trident_version':__version__,
                 'pytorch_version':torch.__version__,
-                'signature':self.signature
+                'signature':self._model.signature
             }, save_path)
 
 
@@ -943,17 +1042,20 @@ class Model(ModelBase):
 
             self._model.eval()
 
-
-        if self.signature is None:
-            self._model.signature = get_signature(self._model.forward)
+        self._model.signature = None
         if "signature" in pretrained_dict.keys():
             self._model.signature = pretrained_dict['signature']
+        if self.signature is None or self.signature !=self._model.signature:
+            self.signature=self._model.signature
 
         self._model.to(torch.device("cuda" if torch.cuda.is_available() else "cpu"))
 
     def summary(self):
         # self.rebinding_input_output(self._model.input_shape)
-        summary(self._model, self.inputs.value_list)
+
+        summary(self._model, [item._shape_tuple[1:] for item in self.inputs.value_list])
+        return self
+
 
     def predict(self,input):
         raise NotImplementedError
@@ -1019,6 +1121,14 @@ class Model(ModelBase):
 
         return sorted(keys)
 
+    def cpu(self):
+        if isinstance(self._model,(nn.Module,Layer)):
+            self._model.to("cpu")
+
+    def cuda(self):
+        if isinstance(self._model, (nn.Module, Layer)):
+            self._model.to("cuda")
+
 
 class ImageClassificationModel(Model):
     def __init__(self, inputs=None,  input_shape=None,output=None):
